/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.jenkins.results.parser;

import java.io.IOException;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Michael Hashimoto
 */
public class LegacyDataArchiveGroup {

	public LegacyDataArchiveGroup(
		LegacyDataArchiveUtil legacyDataArchiveUtil,
		String legacyDataArchiveType, String portalVersion) {

		_legacyDataArchiveUtil = legacyDataArchiveUtil;
		_legacyDataArchiveType = legacyDataArchiveType;
		_portalVersion = portalVersion;
	}

	public void addLegacyDataArchive(LegacyDataArchive legacyDataArchive) {
		_legacyDataArchives.add(legacyDataArchive);
	}

	public void commitLegacyDataArchives() throws IOException {
		for (LegacyDataArchive legacyDataArchive : _legacyDataArchives) {
			if (!legacyDataArchive.isUpdated()) {
				legacyDataArchive.updateLegacyDataArchive();
			}
		}

		GitWorkingDirectory legacyDataGitWorkingDirectory =
			_legacyDataArchiveUtil.getLegacyDataGitWorkingDirectory();

		String status = legacyDataGitWorkingDirectory.status();

		if (!status.contains("nothing to commit") &&
			!status.contains("nothing added to commit")) {

			Commit latestManualCommit =
				_legacyDataArchiveUtil.getLatestManualCommit();

			legacyDataGitWorkingDirectory.commitStagedFilesToCurrentBranch(
				JenkinsResultsParserUtil.combine(
					"archive:ignore Update '", _legacyDataArchiveType,
					"' for '", _portalVersion, "' at ",
					latestManualCommit.getAbbreviatedSHA(), "."));

			String gitLog = legacyDataGitWorkingDirectory.log(1);

			_commit = CommitFactory.newCommit(gitLog);
		}
	}

	public Commit getCommit() {
		return _commit;
	}

	public String getLegacyDataArchiveType() {
		return _legacyDataArchiveType;
	}

	public boolean isUpdated() {
		for (LegacyDataArchive legacyDataArchive : _legacyDataArchives) {
			if (!legacyDataArchive.isUpdated()) {
				return false;
			}
		}

		return true;
	}

	private Commit _commit;
	private List<LegacyDataArchive> _legacyDataArchives = new ArrayList<>();
	private final String _legacyDataArchiveType;
	private final LegacyDataArchiveUtil _legacyDataArchiveUtil;
	private final String _portalVersion;

}